@@ -1,5 +1,5 @@
 /* GConf
- * Copyright (C) 1999, 2000 Red Hat Inc.
+ * Copyright (C) 1999, 2000, 2002 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -13,10 +13,11 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
+#include <config.h>
 #include "gconf-value.h"
 #include "gconf-error.h"
 #include "gconf-schema.h"
@@ -49,10 +50,8 @@ typedef struct {
 static void
 set_string(gchar** dest, const gchar* src)
 {
-  if (*dest != NULL)
-    g_free(*dest);
-
-  *dest = src ? g_strdup(src) : NULL;
+  g_free(*dest);
+  *dest = g_strdup(src);
 }
 
 /*
@@ -63,11 +62,17 @@ GConfValue*
 gconf_value_new(GConfValueType type)
 {
   GConfValue* value;
-
-  g_return_val_if_fail(GCONF_VALUE_TYPE_VALID(type), NULL);
+  static gboolean initted = FALSE;
   
-  /* Probably want to use mem chunks here eventually. */
-  value = (GConfValue*) g_new0 (GConfRealValue, 1);
+  g_return_val_if_fail(GCONF_VALUE_TYPE_VALID(type), NULL);
+
+  if (!initted)
+    {
+      _gconf_init_i18n ();
+      initted = TRUE;
+    }
+  
+  value = (GConfValue*) g_slice_new0 (GConfRealValue);
 
   value->type = type;
 
@@ -84,6 +89,9 @@ gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
 {
   GConfValue* value;
 
+  g_return_val_if_fail (type != GCONF_VALUE_LIST, NULL);
+  g_return_val_if_fail (type != GCONF_VALUE_PAIR, NULL);
+
   value = gconf_value_new(type);
 
   switch (type)
@@ -702,6 +710,18 @@ copy_value_list(GSList* list)
   return copy;
 }
 
+GType
+gconf_value_get_type ()
+{
+  static GType type = 0;
+
+  if (type == 0)
+    type = g_boxed_type_register_static (g_intern_static_string ("GConfValue"),
+                                         (GBoxedCopyFunc) gconf_value_copy,
+                                         (GBoxedFreeFunc) gconf_value_free);
+  return type;
+}
+
 GConfValue* 
 gconf_value_copy (const GConfValue* src)
 {
@@ -799,8 +819,7 @@ gconf_value_free(GConfValue* value)
   switch (real->type)
     {
     case GCONF_VALUE_STRING:
-      if (real->d.string_data != NULL)
-        g_free(real->d.string_data);
+      g_free(real->d.string_data);
       break;
     case GCONF_VALUE_SCHEMA:
       if (real->d.schema_data != NULL)
@@ -820,7 +839,7 @@ gconf_value_free(GConfValue* value)
       break;
     }
   
-  g_free(value);
+  g_slice_free(GConfRealValue, real);
 }
 
 const char*
@@ -966,6 +985,14 @@ gconf_value_set_int(GConfValue* value, gint the_int)
 
 void        
 gconf_value_set_string(GConfValue* value, const gchar* the_str)
+{  
+  gconf_value_set_string_nocopy (value,
+                                 g_strdup (the_str));
+}
+
+void
+gconf_value_set_string_nocopy (GConfValue *value,
+                               char       *str)
 {
   GConfRealValue *real;
 
@@ -973,8 +1000,9 @@ gconf_value_set_string(GConfValue* value, const gchar* the_str)
   g_return_if_fail(value->type == GCONF_VALUE_STRING);
 
   real = REAL_VALUE (value);
-  
-  set_string(&real->d.string_data, the_str);
+
+  g_free (real->d.string_data);
+  real->d.string_data = str;
 }
 
 void        
@@ -1174,7 +1202,6 @@ gconf_value_compare (const GConfValue *value_a,
         return 1;
       else
         return 0;
-      break;
     case GCONF_VALUE_FLOAT:
       if (gconf_value_get_float (value_a) < gconf_value_get_float (value_b))
         return -1;
@@ -1182,11 +1209,9 @@ gconf_value_compare (const GConfValue *value_a,
         return 1;
       else
         return 0;
-      break;
     case GCONF_VALUE_STRING:
       return strcmp (gconf_value_get_string (value_a),
                      gconf_value_get_string (value_b));
-      break;
     case GCONF_VALUE_BOOL:
       if (gconf_value_get_bool (value_a) == gconf_value_get_bool (value_b))
         return 0;
@@ -1195,7 +1220,6 @@ gconf_value_compare (const GConfValue *value_a,
         return 1;
       else
         return -1;
-      break;
     case GCONF_VALUE_LIST:
       {
         GSList *list_a;
@@ -1224,7 +1248,6 @@ gconf_value_compare (const GConfValue *value_a,
         else
           return 0;
       }
-      break;
     case GCONF_VALUE_PAIR:
       {
         GConfValue *a_car, *b_car, *a_cdr, *b_cdr;
@@ -1261,10 +1284,8 @@ gconf_value_compare (const GConfValue *value_a,
 
         return 0;
       }
-      break;
     case GCONF_VALUE_INVALID:
       return 0;
-      break;
     case GCONF_VALUE_SCHEMA:
       {
         const char *locale_a, *locale_b;
@@ -1339,7 +1360,6 @@ gconf_value_compare (const GConfValue *value_a,
 
         return 0;
       }
-      break;
     }
 
   g_assert_not_reached ();
@@ -1366,6 +1386,8 @@ gconf_meta_info_new(void)
 void
 gconf_meta_info_free(GConfMetaInfo* gcmi)
 {
+  set_string(&gcmi->schema, NULL);
+  set_string(&gcmi->mod_user, NULL);
   g_free(gcmi);
 }
 
@@ -1418,6 +1440,29 @@ gconf_meta_info_set_mod_time(GConfMetaInfo* gcmi,
  * GConfEntry
  */
 
+typedef struct {
+  char *key;
+  GConfValue *value;
+  char *schema_name;
+  int refcount;
+  guint is_default : 1;
+  guint is_writable : 1;
+} GConfRealEntry;
+
+#define REAL_ENTRY(x) ((GConfRealEntry*)(x))
+
+GType
+gconf_entry_get_type ()
+{
+  static GType type = 0;
+
+  if (type == 0)
+    type = g_boxed_type_register_static (g_intern_static_string ("GConfEntry"),
+                                         (GBoxedCopyFunc) gconf_entry_ref,
+                                         (GBoxedFreeFunc) gconf_entry_unref);
+  return type;
+}
+
 GConfEntry*
 gconf_entry_new (const char *key,
                  const GConfValue  *val)
@@ -1430,43 +1475,47 @@ gconf_entry_new (const char *key,
 GConfEntry* 
 gconf_entry_new_nocopy (char* key, GConfValue* val)
 {
-  GConfEntry* pair;
+  GConfRealEntry* real;
 
-  pair = g_new (GConfEntry, 1);
+  real = g_slice_new (GConfRealEntry);
 
-  pair->key   = key;
-  pair->value = val;
-  pair->schema_name = NULL;
-  pair->is_default = FALSE;
-  pair->is_writable = TRUE;
-  pair->refcount = 1;
+  real->key   = key;
+  real->value = val;
+  real->schema_name = NULL;
+  real->is_default = FALSE;
+  real->is_writable = TRUE;
+  real->refcount = 1;
   
-  return pair;
+  return (GConfEntry*) real;
 }
 
 void
 gconf_entry_ref (GConfEntry *entry)
 {
   g_return_if_fail (entry != NULL);
-  entry->refcount += 1;
+  
+  REAL_ENTRY (entry)->refcount += 1;
 }
 
 void
 gconf_entry_unref (GConfEntry *entry)
 {
+  GConfRealEntry *real;
+  
   g_return_if_fail (entry != NULL);
-  g_return_if_fail (entry->refcount > 0);
+  g_return_if_fail (REAL_ENTRY (entry)->refcount > 0);
 
-  entry->refcount -= 1;
+  real = REAL_ENTRY (entry);
+  
+  real->refcount -= 1;
 
-  if (entry->refcount == 0)
+  if (real->refcount == 0)
     {
-      g_free (entry->key);
-      if (entry->value)
-        gconf_value_free (entry->value);
-      if (entry->schema_name)
-        g_free (entry->schema_name);
-      g_free (entry);
+      g_free (real->key);
+      if (real->value)
+        gconf_value_free (real->value);
+      g_free (real->schema_name);
+      g_slice_free (GConfRealEntry, real);
     }
 }
 
@@ -1480,12 +1529,15 @@ GConfEntry*
 gconf_entry_copy (const GConfEntry *src)
 {
   GConfEntry *entry;
-
-  entry = gconf_entry_new (src->key, src->value);
-
-  entry->schema_name = g_strdup (src->schema_name);
-  entry->is_default = src->is_default;
-  entry->is_writable = src->is_writable;
+  GConfRealEntry *real;
+  
+  entry = gconf_entry_new (REAL_ENTRY (src)->key,
+                           REAL_ENTRY (src)->value);
+  real = REAL_ENTRY (entry);
+  
+  real->schema_name = g_strdup (REAL_ENTRY (src)->schema_name);
+  real->is_default = REAL_ENTRY (src)->is_default;
+  real->is_writable = REAL_ENTRY (src)->is_writable;
 
   return entry;
 }
@@ -1494,29 +1546,35 @@ gboolean
 gconf_entry_equal (const GConfEntry *a,
                    const GConfEntry *b)
 {
+  GConfRealEntry *real_a;
+  GConfRealEntry *real_b;
+  
   g_return_val_if_fail (a != NULL, FALSE);
   g_return_val_if_fail (b != NULL, FALSE);
 
+  real_a = REAL_ENTRY (a);
+  real_b = REAL_ENTRY (b);
+  
   /* do the cheap checks first, why not */
-  if (a->value && !b->value)
+  if (real_a->value && !real_b->value)
     return FALSE;
-  else if (!a->value && b->value)
+  else if (!real_a->value && real_b->value)
     return FALSE;
-  else if (a->is_default != b->is_default)
+  else if (real_a->is_default != real_b->is_default)
     return FALSE;
-  else if (a->is_writable != b->is_writable)
+  else if (real_a->is_writable != real_b->is_writable)
     return FALSE;
-  else if (strcmp (a->key, b->key) != 0)
+  else if (strcmp (real_a->key, real_b->key) != 0)
     return FALSE;
-  else if (a->schema_name && !b->schema_name)
+  else if (real_a->schema_name && !real_b->schema_name)
     return FALSE;
-  else if (!a->schema_name && b->schema_name)
+  else if (!real_a->schema_name && real_b->schema_name)
     return FALSE;
-  else if (a->schema_name && b->schema_name &&
-           strcmp (a->schema_name, b->schema_name) != 0)
+  else if (real_a->schema_name && real_b->schema_name &&
+           strcmp (real_a->schema_name, real_b->schema_name) != 0)
     return FALSE;
-  else if (a->value && b->value &&
-           gconf_value_compare (a->value, b->value) != 0)
+  else if (real_a->value && real_b->value &&
+           gconf_value_compare (real_a->value, real_b->value) != 0)
     return FALSE;
   else
     return TRUE;
@@ -1525,8 +1583,8 @@ gconf_entry_equal (const GConfEntry *a,
 GConfValue*
 gconf_entry_steal_value (GConfEntry* entry)
 {
-  GConfValue* val = entry->value;
-  entry->value = NULL;
+  GConfValue* val = REAL_ENTRY (entry)->value;
+  REAL_ENTRY (entry)->value = NULL;
   return val;
 }
 
@@ -1535,7 +1593,7 @@ gconf_entry_get_key (const GConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, NULL);
 
-  return entry->key;
+  return REAL_ENTRY (entry)->key;
 }
 
 GConfValue*
@@ -1543,7 +1601,7 @@ gconf_entry_get_value (const GConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, NULL);
 
-  return entry->value;
+  return REAL_ENTRY (entry)->value;
 }
 
 const char*
@@ -1551,7 +1609,7 @@ gconf_entry_get_schema_name (const GConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, NULL);
 
-  return entry->schema_name;
+  return REAL_ENTRY (entry)->schema_name;
 }
 
 gboolean
@@ -1559,7 +1617,7 @@ gconf_entry_get_is_default  (const GConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, FALSE);
 
-  return entry->is_default;
+  return REAL_ENTRY (entry)->is_default;
 }
 
 gboolean
@@ -1567,7 +1625,7 @@ gconf_entry_get_is_writable (const GConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, FALSE);
 
-  return entry->is_writable;
+  return REAL_ENTRY (entry)->is_writable;
 }
 
 
@@ -1583,34 +1641,33 @@ void
 gconf_entry_set_value_nocopy(GConfEntry* entry,
                              GConfValue* val)
 {
-  if (entry->value)
-    gconf_value_free(entry->value);
+  if (REAL_ENTRY (entry)->value)
+    gconf_value_free (REAL_ENTRY (entry)->value);
 
-  entry->value = val;
+  REAL_ENTRY (entry)->value = val;
 }
 
 void
 gconf_entry_set_schema_name(GConfEntry* entry,
                             const gchar* name)
 {
-  if (entry->schema_name)
-    g_free(entry->schema_name);
+  g_free (REAL_ENTRY (entry)->schema_name);
 
-  entry->schema_name = name ? g_strdup(name) : NULL;
+  REAL_ENTRY (entry)->schema_name = g_strdup(name);
 }
 
 void
 gconf_entry_set_is_default (GConfEntry* entry,
                             gboolean is_default)
 {
-  entry->is_default = is_default;
+  REAL_ENTRY (entry)->is_default = is_default;
 }
 
 void
 gconf_entry_set_is_writable (GConfEntry  *entry,
                              gboolean     is_writable)
 {
-  entry->is_writable = is_writable;
+  REAL_ENTRY (entry)->is_writable = is_writable;
 }
 
 
@@ -1620,6 +1677,8 @@ gconf_value_validate (const GConfValue *value,
 {
   GConfRealValue *real;
 
+  g_return_val_if_fail (value != NULL, FALSE);
+
   real = REAL_VALUE (value);
   
   switch (value->type)
